% Make movies of various signal representations aligned with audio
% Visualize modspec, etc
%
% v2. Replaced spectrotemporal modspec with modfreq x freq
% v3. Temporal envelope resolution increased from 50 -> 100 samps/sec
% v4. Magnitude as one big surf plot with shading interp'd
% v5. Using 2-second analysis windows
% v6. Modulation filterbank (constant-Q), 1-sec analysis windows
%     Temporal envelope resolution increased from 100 -> 200 samps/sec

% KJPW 2-10-2017

clear all; clc; close all

d_sr = 20000; % input is downsampled to 10k, but the original audio is attached to the video

modspec_viewangle = [11.000 80.0000]; % viewing angle for the top panel (modspec)


%% Get soundfile

input_sound = input('Audio name? ', 's');
sound_index = dir(strcat(input_sound,'*')); 
[soundfile_str, sr] = audioread(sound_index.name);
n_chan = size(soundfile_str,2);
secs_start = input('Start at beginning (0), or enter the start point in secs: ');
secs_use = input('Enter the end point in secs: ');

if secs_use > length(soundfile_str)/sr
    secs_use = floor(length(soundfile_str)/sr);
end

if size(soundfile_str,1)>sr*secs_use
  soundfile_str = soundfile_str(sr*secs_start+1:sr*secs_use,:); %
  soundfile_str(end-sr+1:end,:) = soundfile_str(end-sr+1:end,:).*repmat(((sr-1:-1:0)./(sr-1)),n_chan,1)'; % 1-sec fadeout
  if secs_start~=0
      soundfile_str(1:sr,:) = soundfile_str(1:sr,:).*repmat(((1:sr)./(sr)),n_chan,1)'; % 1-sec fadein
  end
end

orig_audio = sum(soundfile_str,2)./2; osr = sr;

if sr~=d_sr
   soundfile_str=resample(soundfile_str,d_sr,sr); % resample to desired output rate
   sr=d_sr;
end

soundfile_str = soundfile_str * (.9/max(max(abs(soundfile_str)))); % peak-normalize to 0.9

soundfile=sum(soundfile_str,2)./2; % make it mono


%% Take cochleagram of the entire signal

analysis_win_dur = 2000; % ms

soundfile = [zeros(sr*analysis_win_dur/1000,1); soundfile; zeros(sr*analysis_win_dur/1000,1)];

if size(soundfile_str,2)>1
 soundfile_str = [zeros(sr*analysis_win_dur/1000,2); soundfile_str; zeros(sr*analysis_win_dur/1000,2)];
else
 soundfile_str = [zeros(sr*analysis_win_dur/1000,1); soundfile_str; zeros(sr*analysis_win_dur/1000,1)];
end

soundfile_dur = length(soundfile)/sr; % secs

frame_rate = 50; % ms/frame
fps = (1000/frame_rate);
samps_per_frame = sr/fps;
n_frames = floor(soundfile_dur*fps);
pre_dur = 500; % ms
pre_dur_samps = pre_dur * sr / 1000 ;
cur_aud_samp = 1; 

fprintf('Taking subband envelopes ... \n');
[subbands, subband_envs] = erbcochleagram_200hz_env(soundfile,d_sr);
coch = subband_envs';
cochsize = size(coch,2);
cframes_per_sec = cochsize/soundfile_dur;
cframes_per_frame = cframes_per_sec * (frame_rate/1000);


%% Loop over analysis frames
close all;
% hFig = figure(1); set(hFig, 'Position', [50 50 750 1000], 'Color', [1 1 1]);
jFig = figure(2); set(jFig, 'Position', [700 50 750 1000], 'Color', [1 1 1]);
set(gcf, 'Renderer', 'zbuffer')

vidname = strcat('ModSpec_movie_',input_sound,'_.avi');
videoFWriter = vision.VideoFileWriter(vidname,'FrameRate',20,'AudioInputPort',1);

lock=0; zlock=0; plock=0; pzlock=0; % z-axis / color lock
jlock=0; jzlock=0; jplock=0; jpzlock=0; % z-axis / color lock
for frame = 1:n_frames-((sr*analysis_win_dur/1000)/samps_per_frame)-10
    
%     figure(1);clf; 
    figure(2); clf;
    
    fprintf ('Processing frame %d ... \n',frame);
    start_smp = samps_per_frame*(frame-1) +1;
    end_smp = start_smp + (sr*analysis_win_dur/1000) -1;
    soundseg = hannn(soundfile(start_smp:end_smp)',50,sr)';
    
    if end_smp+pre_dur_samps < length(soundfile)
        
        pastseg = hannn(soundfile(end_smp:end_smp+pre_dur_samps)',50,sr)';
        
        figure(2);
        subplot(100,100,subplot_calc(85,1,95,10)); plot(flipud(pastseg),'k');
        axis tight; ylim([-1 1]);
        set(gca,'XTick',[],'XTickLabel',[])
        xlabel('Time')
        title {Arriving Wave}
    end
    
    
    %% cochleagram
    
    subplot(100,100,subplot_calc(80,11,100,100));
    
    cc = coch(:,(ceil(cframes_per_frame*(frame-1)+1)):ceil(cframes_per_frame*(frame-1)+(cframes_per_sec*analysis_win_dur/1000)));
    cc = fliplr(cc);
    imagesc(flipud(cc)); 
     colormap(gca,'jet');
    caxis([0 0.9]);
    
    set(gca,'YTick',[1 size(cc,1)],'YTickLabel',['10k';'0  '])
    ylabel(''); xlabel('Cochleagram')
    set(gca,'XTick',[],'XTickLabel',[])
    freezeColors
    hold on;
    
    

    %% Modulation frequency vs audio frequency (NON BIOL MOD FILT)
%     figure(1);
%      subplot(100,100,subplot_calc(4,1,10,10)); axis off
%      title({'Modulation Frequency (X-axis)';'';'by Cochlear Channel(Y-axis)'})
%     
%     subplot(100,100,subplot_calc(1,12,42,100));  
%     
%     YY = abs(fft(cc,size(cc,2),2)) / size(cc,2);
%     ZZ = YY(:,1:size(cc,2)/2+1);
%     ZZ = 2*(ZZ(:,2:end-1));
%     
%     modfreq = log(ZZ-min(min(ZZ)));
%     
%     ii = flipud(modfreq);
%     ii(find(ii<-16)) =-16;
%     norm_alph = (ii - min(min(ii))) / ( max(max(ii)) - min(min(ii)) );
%     alpha(norm_alph); 
%     
%     if frame>1 % denoise 
%         dmf = abs(modfreq-past_mf);
%         bgr = (mean(mean(dmf)));
%         modfreq(find(dmf>bgr)) = past_mf(find(dmf>bgr));
%     end
%     
%     surf(modfreq,'MeshStyle','row','LineWidth',0.3); colormap jet;
%     shading interp
%     
%     if frame > fps*2 && mean(rms(soundfile_str(end_smp:end_smp+samps_per_frame,:)))>0.05 && lock==0
%        c1cax = get(gca,'clim'); 
%        lock = 1;
%     elseif frame>fps*2 && lock==1
%        set(gca,'clim',([c1cax]));
%     end
%     
%      if frame > fps*2 && mean(rms(soundfile_str(end_smp:end_smp+samps_per_frame,:)))>0.05  && zlock==0
%        m1zax = get(gca,'zlim'); 
%        zlock=1;
%      elseif frame>fps*2 && zlock ==1;
%         zlim([m1zax]);hold on
%     end
%     
%     set(gca,'Color', [1 1 1]);
%     title({'Magnitude, 0-100Hz'})
%     set(gca,'XTick',[1 100],'XTickLabel',['0    ';'100Hz'])
%     set(gca,'YTick',[1 size(cc,1)],'YTickLabel',[])
%     xlim([0 100]);
%     ylim([0 size(cc,1)]);
%     box off
%     set(gca,'ZTick',[]);
%     set(gca,'View',[11.000 80.0000], 'LineWidth',0.01);
%     freezeColors
%     hold on
%     
%     
%     past_mf = modfreq;
%     
   
%     %% Broadband Modulation Spectrum
%      subplot(100,100,subplot_calc(43,10,53,84));
%      plot(mean(modfreq,1),'k', 'LineWidth',2.3);
%      xlim([0 100]);
%      set(gca,'XTick',[1 100],'XTickLabel',['0    ';'100Hz'])
%      ylim([-8 0]);
%      set(gca,'YTick',[],'Box','off'); hold on;
%     
%      
%      
     %% Modspec phase
     
%    
%     YY = imag(fft(cc,size(cc,2),2)) / size(cc,2);
%     ZZ = YY(:,1:size(cc,2)/2+1);
%     ZZ = (ZZ(:,2:end-1));
%     
%     modfreq = (ZZ); 
%     
%     
%     subplot(100,100,subplot_calc(59,43,75,100));
%     
%     surf(modfreq,'LineWidth',.001);
%     shading interp
%     if frame > fps*2 && mean(rms(soundfile_str(end_smp:end_smp+samps_per_frame,:)))>0.05 && plock==0
%        c3cax = get(gca,'clim'); 
%        plock=1;
%     elseif frame>fps*2 && plock==1
%       set(gca,'clim',([c3cax]));
%     end
%     
%      if frame == fps*2 && mean(rms(soundfile_str(end_smp:end_smp+samps_per_frame,:)))>0.05 && pzlock==0
%        m2zax = get(gca,'zlim'); 
%        pzlock=1;
%      elseif frame>fps*2 && pzlock==1
%         zlim([m2zax]);hold on
%     end
%     
%     set(gca,'XTick',[1 100],'XTickLabel',['0   ';'50Hz'])
%     set(gca,'YTick',[]);
%     xlim([0 100]);
%     ylim([0 size(cc,1)]);
%      set(gca,'ZTick',[],'ZTickLabel','')
%     set(gca,'View',[11.000 80.0000]);hold on;
%     set(gca,'Color', [1 1 1]);
%      title ('Phase, 0-50Hz');
%     colormap HSV
%     
%     
    %% Modspec using biologically-plausible modulation filters
    
    N = 50; % number of filters
    esr = 200;% envelope sample rate
    low_lim = 2; % center of lowest mod filter in Hz
    hi_lim = (esr/2); % center of highest mod filter in Hz
    Q = 1.2;
    ccm=cc;
    
    winlen = [2 4 8];
    mpc=zeros(64,50);
    for len = 1:length(winlen)
    clearvars mp    
    signal_length = floor(length(ccm)/winlen(len));
    
    [mod_filts, Cfs, freqs] = make_constQ_cos_filters(signal_length, esr, N, low_lim, hi_lim, Q);
    
%     cc_weighted = repmat(cos(0:((pi/2)/size(cc,2)):(pi/2)-((pi/2)/size(cc,2)))',1,size(cc,1))' .* cc;
%     cc_weighted = repmat((1:(-1/size(cc,2)):(0+(1/size(cc,2))))',1,size(cc,1))' .* cc;
    
    use_zp = 0;
    for sub = 1:size(ccm,1)
        mp(sub,:) = stat_mod_power_win(ccm(sub,1:signal_length)', mod_filts, use_zp);
        mp(sub,:) = mp(sub,:).* (rms(ccm(sub,1:signal_length)'));
    end
    
    mpc = mpc + log10(mp+0.1);
    
    end
    mp = mpc./length(winlen);
    
    subplot(100,100,subplot_calc(4,1,10,10)); axis off
    title({'Log Modulation Frequency (X-axis)';'';'by Cochlear Channel(Y-axis)'})
    
    subplot(100,100,subplot_calc(1,12,42,100));  
    
    zmp = zscore(mp) + 2;
    [a b] = find(zmp==min(min(zmp)));
    
    if length(a) > 1
        a = a(1);
        b = b(1);
    end
    
    mp_min = mp(a,b);
    
%     if frame>1 % denoise 
%         dmp = abs(mp-past_mp);
%         bgr = (mean(mean(dmp)));
%         mp(find(dmp>bgr)) = past_mp(find(dmp>bgr));
%     end
   

    surf(mp,'MeshStyle','row','LineWidth',0.3); colormap jet; shading interp
    
    %% Lock the axes after 2 seconds, as long as its not silent
    if frame > fps*2 && mean(rms(soundfile_str(end_smp:end_smp+samps_per_frame,:)))>0.05 && jlock==0
       jc1cax = get(gca,'clim'); 
       jlock = 1;
    elseif frame>fps*2 && jlock==1
       set(gca,'clim',([jc1cax]));
    end
    
     if frame > fps*2 && mean(rms(soundfile_str(end_smp:end_smp+samps_per_frame,:)))>0.05  && jzlock==0
       jm1zax = get(gca,'zlim'); 
       jzlock=1;
     elseif frame>fps*2 && jzlock ==1;
        zlim([jm1zax]);hold on
    end
    
    set(gca,'Color', [1 1 1]);
    title({'Magnitude, 1-100Hz'})
    Cfss = num2str(Cfs(1:5:end)');
    set(gca,'XTick',[1:5:length(Cfs)],'XTickLabel',Cfss(:,1:4))
    set(gca,'YTick',[1 size(cc,1)],'YTickLabel',[])
    xlim([1 N]);
    ylim([1 size(cc,1)]);
    box off
    set(gca,'ZTick',[]);
    set(gca,'View',modspec_viewangle, 'LineWidth',0.01);
    freezeColors
    
%     ii = mp; %flipud(mp);
%     ii = 1 - (ii * -1); 
%     ii(zscore(ii)>-1) = 1;
%     norm_alph = .5 + ((ii - min(min(ii))) / (2*( max(max(ii)) - min(min(ii)))));
%     alpha(norm_alph); 

    hold on
    
    past_mp = mp;
    
        %% Broadband Modulation Spectrum
     subplot(100,100,subplot_calc(53,10,63,84));
     plot(mean(mp,1),'k', 'LineWidth',2.3);
     xlim([1 N]);
     set(gca,'XTick',[1:5:length(Cfs)],'XTickLabel',Cfss(:,1:4))
     set(gca,'YTick',[],'Box','off'); hold on;
    
    
    %% Save frame into .AVI and attach audio
    pause(0.075)
    endaudfr = floor(cur_aud_samp+(samps_per_frame*osr/sr));
    if endaudfr < length(orig_audio)
        AudioFrame = orig_audio(cur_aud_samp:endaudfr);
        cur_aud_samp = floor(cur_aud_samp+(samps_per_frame*osr/sr));
        Fr = getframe(jFig);
        step(videoFWriter,Fr.cdata,AudioFrame)
    else
        release(videoFWriter);
    end
    
    
  
end

release(videoFWriter);

